//=============================================================================
// QLModeClone.
//
// Author: Francesco Biscazzo
// Date: 2016
//=============================================================================
class QLModeClone extends AdvancedMutator;

function Fired(Pawn pawn, Weapon weapon, Actor HitActor) {
	if (HitActor != none) {
		ToggleTriggerActor(pawn, HitActor, false);
	}
}

function ChangedWeaponState(Pawn pawn, Weapon weapon, String oldState, String newState) {
	if (newState ~= "idle")
		UnTriggerActors(pawn);
}

function ChangedZone(Pawn pawn, ZoneInfo oldZone, ZoneInfo newZone) {
	local Inventory nextInv;
	local int i, j;
	local String nextZoneName;
	local bool bCanPass;
	
	if (FindSplittedStr(newZone.ZoneName, "_", 0, false) ~= "key") {
		nextInv = pawn.Inventory;
		while (nextInv != none) {
			// has a key.
			if (nextInv.IsA('QLKeyClone')) {
				for (i = 0; i < 10; i++) {
					for (j = 1; j < 20; j++) {
						if (QLKeyClone(nextInv).ZoneNames[i] != "") {
							nextZoneName = FindSplittedStr(newZone.ZoneName, "_", j, false);
							
							if (nextZoneName != "") {
								if (QLKeyClone(nextInv).ZoneNames[i] ~= nextZoneName) {
									//the key allow the player to pass in this zone.
									bCanPass = true;
								}
							}
						}
					}
				}
			}
			
			nextInv = nextInv.Inventory;
		}
		
		if (!bCanPass) {
			pawn.Health = -10000; //make sure gibs
			pawn.Died(None, 'Gibbed', pawn.Location);
			
			if (pawn.IsA('PlayerPawn')) {
				PlayerPawn(pawn).ClientMessage("You shall not pass!");
			}
		}
	}
}

function ToggleTriggerActor(Pawn pawn, Actor actor, bool bUnTrigger) {
	local Inventory nextInv;
	local int i, j;
	local String nextTag;
	
	if (FindSplittedStr(String(actor.Tag), "_", 0, false) ~= "key") {
		nextInv = pawn.Inventory;
		while (nextInv != none) {
			if (nextInv.IsA('QLKeyClone')) {
				// has a key.
				for (i = 0; i < 10; i++) {
					for (j = 1; j < 20; j++) {
						if (QLKeyClone(nextInv).Events[i] != "") {
							nextTag = FindSplittedStr(String(actor.Tag), "_", j, false);
							
							if (nextTag != "") {
								if (QLKeyClone(nextInv).Events[i] ~= nextTag) {
									//the key can trigger actors with this tag.
									if (!bUnTrigger) {
										actor.Trigger(pawn, pawn);
									}
									else {
										actor.UnTrigger(pawn, pawn);
									}
								}
							}
						}
					}
				}
			}
			
			nextInv = nextInv.Inventory;
		}
	}
}

function UnTriggerActors(Pawn pawn) {
	local Actor A;

	foreach AllActors(class'Actor', A) {
		ToggleTriggerActor(pawn, A, true);
	}
}

/* Took from CommandSystem */
/*
 * Split a string and return the string from the given position. If no string was found in the
 * given position, return an empty string.
 *
 * @str = the string to split.
 * @div = the string to interpret as divider.
 * @num = the position of the string to return.
 * @bDiv = true to keep dividers, false to remove dividers.
 */
static function String FindSplittedStr(String str, String div, int num, bool bDiv) {
   local bool bEOL;
   local String tempChar;
   local int precount, curcount, wordcount, ParametersLength;
   
   local String word;
   
   if (str != "") {
		if (InStr(str, div) != -1) {
			ParametersLength = Len(str);
			bEOL = false;
			precount = 0;
			curcount = 0;
			wordcount = 0;
			
			while(!bEOL) {
				tempChar = Mid(str, curcount, 1); //go up by 1 count
				if(tempChar != div)
					curcount++;
				else if(tempChar == div) {
					if (wordcount == num) {
						word = Mid(str, precount, curcount - precount);
						break;
					}
					wordcount++;
					if(bDiv)
						precount = curcount; //leaves the divider
					else
						precount = curcount + 1; //removes the divider.
					curcount++;
				}
				
				//end of str string.
				if(curcount == ParametersLength) {
					if (wordcount == num)
						word = Mid(str, precount);
					bEOL = true;
				}
			}
		}
		else
			word = str;
	}
	
	return word;
}